#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>

#include "adc.h"
#include "macros.h"
#include "serial.h"

void adc_init(void)
{
    // The pot voltage is a high output!
    set(DDRB, PB5);
    
    // Prepare potentiometer input and output
    ADMUX = 0b01000110; // AVcc reference, ADC6 as input
    ADCSRA = 0b10011011; // Enabled, flag clear, no auto run, prescaler of 8
}

int adc_read(void)
{
    set(PORTB, PB5); // Put a voltage across the potentiometer

    set_sleep_mode(SLEEP_MODE_ADC);
    sleep_enable();
    sleep_cpu();
    sleep_disable();
    
    while(bit_test(ADCSRA, ADSC));
    clear(PORTB, PB5); // Remove the voltage across the potentiometer.
    
    int val = ADCL; // We have to read ADCL before ADCH
    val += ADCH << 8;
    return val;
}

int adc_read_low(void)
{
    if (!bit_test(ADMUX, REFS1)) {
        set(ADMUX, REFS1);
        _delay_ms(15);
        set(ADCSRA, ADSC); // Dummy read
        while(bit_test(ADCSRA, ADSC));
    }
    return adc_read();
}
    
int adc_read_high(void)
{
    if (bit_test(ADMUX, REFS1)) {
        clear(ADMUX, REFS1);
        _delay_ms(15);
        set(ADCSRA, ADSC); // Dummy read
        while(bit_test(ADCSRA, ADSC));
    }
    return adc_read();
}

int adc_read_combined(void)
{
    int l = adc_read_low();
    int h = adc_read_high();
    return l + h;
}

ISR(ADC_vect)
{
    ; // Nothing to do here.
}